"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Request {
    constructor(event, context, routeParams) {
        this.event = event;
        this.context = context;
        this.routeParams = routeParams;
    }
    /**
     * Gets the query string parameters from the event.
     *
     * @return {Object}
     * An object containing all the query string parameters.
     */
    get query() {
        return this.routeParams || this.event.queryStringParameters || {};
    }
    /**
     * Returns the inputted pathParameters from the event object.
     *
     * @return {Object}
     * The path parameters object.
     */
    get params() {
        return this.event.pathParameters || {};
    }
    /**
     * Gets the JSON parsed body from the request. If the body is not parse-able an empty object is
     * returned.
     *
     * @return {Object}
     * JSON parsed body input.
     */
    get body() {
        let body;
        try {
            body = JSON.parse(this.event.body);
        }
        catch (error) {
            body = {};
        }
        return body;
    }
    /**
     * Gets the cognito authorizer claims.
     *
     * @return {Object}
     * The lambda context object
     */
    get claims() {
        return this.event.requestContext &&
            this.event.requestContext.authorizer &&
            this.event.requestContext.authorizer.claims
            ? this.event.requestContext.authorizer.claims
            : {};
    }
    /**
     * Gets the whole lambda context object.
     *
     * @return {Object}
     * The lambda context object
     */
    get contextObject() {
        return this.context;
    }
    /**
     * Gets the whole lambda event object.
     *
     * @return {Object}
     * The lambda event object
     */
    get eventObject() {
        return this.event;
    }
    /**
     * Gets the stage variables from the event object.
     *
     * @return {Object}
     * Returns the stage variables if they are available. Will return an empty object if not.
     */
    get stageVariables() {
        return this.event.stageVariables || {};
    }
    /**
     * Returns the raw body sent in the request without it being JSON parsed.
     *
     * @return {string}
     * The raw body from the request.
     */
    get rawBody() {
        return this.event.body;
    }
    /**
     * Return the headers object or an empty object from the event.
     *
     * @return {Object}
     * The headers from the request
     */
    get headers() {
        return this.event.headers || {};
    }
}
exports.default = Request;
//# sourceMappingURL=request.js.map