"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const HttpStatus = require("http-status-codes");
const http_header_fields_typed_1 = require("http-header-fields-typed");
class Response {
    constructor(callback, enableCors = true) {
        this._sent = false;
        this._callback = callback;
        // defaults
        this._statusCode = HttpStatus.OK;
        this._base64Encoded = false;
        if (enableCors) {
            this._headers = {
                'Access-Control-Allow-Origin': '*',
                'Access-Control-Allow-Credentials': true // Required for cookies, authorization headers with HTTPS
            };
        }
        else {
            this._headers = {};
        }
        this._body = null;
    }
    get sent() {
        return this._sent;
    }
    /**
     * Set status `code`.
     *
     * @param code
     */
    status(code) {
        this._statusCode = code;
        return this;
    }
    base64Encoded(value) {
        this._base64Encoded = value;
        return this;
    }
    /**
     * Set header `field` to `val`, or pass
     * an object of header fields.
     *
     * Examples:
     *
     *    res.set('Foo', ['bar', 'baz']);
     *    res.set('Accept', 'application/json');
     *
     * Aliased as `res.header()`.
     */
    set(field, value) {
        this._headers[field] = value;
        return this;
    }
    header(field, value) {
        return this.set(field, value);
    }
    /**
     * Get value for header `field`.
     *
     * @param field
     */
    get(field) {
        return this._headers[field];
    }
    contentType(type) {
        return this.set(http_header_fields_typed_1.default.CONTENT_TYPE, type);
    }
    /**
     * Set the location header to `url`.
     *
     * The given `url` can also be the name of a mapped url, for
     * example by default express supports "back" which redirects
     * to the _Referrer_ or _Referer_ headers or "/".
     *
     * Examples:
     *
     *    res.location('/foo/bar').;
     *    res.location('http://example.com');
     *
     * @param url
     */
    location(url) {
        return this.set(http_header_fields_typed_1.default.LOCATION, url);
    }
    /**
     * Redirect to the given `url` with optional response `status`
     * defaulting to 302.
     *
     * The resulting `url` is determined by `res.location()`, so
     * it will play nicely with mounted apps, relative paths,
     * `"back"` etc.
     *
     * Examples:
     *
     *    res.redirect('/foo/bar');
     *    res.redirect('http://example.com');
     *    res.redirect(301, 'http://example.com');
     *    res.redirect('http://example.com', 301);
     *    res.redirect('../login'); // /blog/post/1 -> /blog/login
     */
    redirect(url, status) {
        this.location(url).status(status || HttpStatus.MOVED_TEMPORARILY);
    }
    error(error, logErrors) {
        let content = '';
        switch (this.get(http_header_fields_typed_1.default.CONTENT_TYPE)) {
            case 'text/plain':
            case 'text/html':
                content = 'Internal Server Error.';
                break;
            case 'application/json':
            default:
                content = JSON.stringify({
                    message: 'Internal Server Error'
                });
        }
        if (logErrors) {
            console.error('Unhandled exception: ', error);
        }
        return this._callback(null, {
            statusCode: HttpStatus.INTERNAL_SERVER_ERROR,
            isBase64Encoded: false,
            body: content,
            headers: this._headers
        });
    }
    send(body) {
        if (this._sent) {
            return;
        }
        let content = '';
        let responseBody = body || this._body || {};
        switch (this.get(http_header_fields_typed_1.default.CONTENT_TYPE)) {
            case 'text/plain':
            case 'text/html':
                content = responseBody.toString();
                break;
            case 'application/json':
            default:
                content = JSON.stringify(responseBody);
        }
        this._sent = true;
        return this._callback(null, {
            statusCode: this._statusCode,
            headers: this._headers,
            isBase64Encoded: this._base64Encoded,
            body: content
        });
    }
}
exports.default = Response;
//# sourceMappingURL=response.js.map